;nyquist plug-in
;version 1
;type process
;categories "http://lv2plug.in/ns/lv2core/#ReverbPlugin"
;name "Reverb 1p..."
;action "Reverberating..."
;info "by Steve Daulton. Released under GPL v2.\nAn implementation of reverb.lsp by Roger Dannenberg. // Presets by Win Bent (whbjr)\n\nThe damping. low cut filter and stereo width act only on the\nreverberation and not on the dry signal.\n\nSelect 'Use Custom' then set the slider values, or select a preset  (slider values are then ignored).\nChoose 'Debug' to see the values that were applied. \n\n"

;; reverb1.ny by Steve Daulton. July 2011.
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html .

;control preset "Presets" choice "Use Custom,Basic (default),Light and Bright,Small and Harsh,Medium Hall,Long and Mild,Cathedral" 0
;control time "Reverberation time" real "seconds" 2.5 0.1 10
;control damping "High frequency damping" real "%" 50 0 100
;control low-cut "Low cut filter" real "Hz" 200 0 1000
;control width "Stereo Width" real "%" 40 0 100
;control mix "Reverb mix proportion" real "%" 20 0 100

;; Create list of values from selected preset or from the slider values
(setq channels
	(case preset
	  (1 (list  2.5   50    200  40  20)) ; Basic (default)
	  (2 (list  0.4   33.3 1000  15  25)) ; Light and Bright
	  (3 (list  0.2   10    800   0  60)) ; Small and Harsh
	  (4 (list  3      0    200  40  20)) ; Medium Hall
	  (5 (list  6.67  80    120  50  50)) ; Long and Mild
	  (6 (list 10      0    200  90  80)) ; Cathedral
	  (T (list time damping low-cut width mix ))))

; Print settings to debug window	  
(format T "Reverb Time:~a sec ~%HF Damping:~a %~%Low cut:~a Hz ~%Width:~a %~%Mix:~a %~%\n"
(nth 0 channels)(nth 1 channels)(nth 2 channels)(nth 3 channels)(nth 4 channels))

;copy channel to vars, limit values to valid range, convert %-ages to Fractions
(setq time (max 0.1 (nth 0 channels)))
(setq damping (max 0 (min 1 (/ (nth 1 channels) 100.0))))
(setq low-cut (max 0 (min (/ *sound-srate* 2.0) (nth 2 channels))))
(setq width (max 0 (min 1 (/ (nth 3 channels) 100.0))))
(setq mix (max 0 (min 1 (/ (nth 4 channels) 100.0))))

;; create new Left channel
(defun Lsplit (leftch df)
  (let((f1 (lp leftch 300)) ;Common Sub below 300
        (f2 (feedback-delay(lp(hp leftch 300)600)(* df 0.2) 0)) ;Low on LEFT 300-600
        (f3 (lp(hp leftch 600)1200)) ;Common MID - 600-1200
        (f4 (feedback-delay(lp(hp leftch 1200)2400)(* df 0.08) 0)) ;High Mid on LEFT 1200-2400
        (f5 (hp  leftch 2400))) ;Common Presence
    (sim f1 f2 f3 f4 f5))) ;mix sounds
    
;; create new Right channel
(defun Rsplit (rightch df)
  (let((f1 (lp rightch 300)) ;Common Sub below 300
        (f2 (feedback-delay(lp(hp rightch 300)600)(* df 0.14) 0)) ;Low Mid on RIGHT 300-600 Hz
        (f3 (lp(hp rightch 600)1200)) ;Common MID - 600-1200
        (f4 (feedback-delay(lp(hp rightch 1200)2400)(* df 0.06) 0)) ;Treble on RIGHT 1200-2400
        (f5 (hp rightch 2400))) ;Common Presence
    (sim f1 f2 f3 f4 f5))) ;mix sounds

(defun stereo (s-in w)
	(let ((dlay (+(* w 0.5)0.2)))
		(if (arrayp s-in)
		(vector
			(sim 
				(mult w (Lsplit (aref s-in 0) dlay))
				(mult w -0.5 (Rsplit (aref s-in 1) dlay))
				(mult (- 1 w) (aref s-in 0)))
			(sim 
				(mult w -0.5 (Lsplit (aref s-in 0) dlay))
				(mult w (Rsplit (aref s-in 1) dlay))
				(mult (- 1 w)(aref s-in 1))))
		s-in)))


;; Reverb function by Roger Dannenberg
(defun reverb (x time) 
  (multichan-expand #'reverb-mono x time))

(defun reverb-mono (ga irevfactor)
  (let (sr ilowpass idel ihz icsc acomball allp1 allp2 allp3 alow allp4 allp5
        arevout)
    (setf sr (snd-srate ga))

    (setf ilowpass 9000.000)       ; frequency of lowpass filter

    (setf idel (list ; list of frequency/delay values
                    (/ 1237.000 sr) (/  1381.000 sr) (/ 1607.000 sr)
                    (/ 1777.000 sr) (/ 1949.000 sr) (/  2063.000 sr)
                    (/ 307.000 sr) (/ 97.000 sr) (/ 71.000 sr)
                    (/ 53.000 sr) (/ 47.000 sr) (/ 37.000 sr)
                    (/ 31.000 sr)))
    ; Nyquist's comb filter uses Hz rather than delay as parameter,
    ; so take reciprocals to get Hz:
    (setf ihz (mapcar #'/ idel))

    (setf icsc (list ; list of delay times
                    (* irevfactor 0.822) (* irevfactor 0.802)
                    (* irevfactor 0.773) (* irevfactor 0.753)
                    (* irevfactor 0.753) (* irevfactor 0.753)
                    (* irevfactor 0.7)))

    (setf acomball (sum
                        (comb ga (nth 0 icsc) (nth 0 ihz))
                        (comb ga (nth 1 icsc) (nth 1 ihz))
                        (comb ga (nth 2 icsc) (nth 2 ihz))
                        (comb ga (nth 3 icsc) (nth 3 ihz))
                        (comb ga (nth 4 icsc) (nth 4 ihz))
                        (comb ga (nth 5 icsc) (nth 5 ihz))))

    (setf allp1 (alpass acomball (nth 6 icsc) (nth 6 ihz)))
    (setf allp2 (alpass allp1 (nth 6 icsc) (nth 7 ihz)))
    (setf allp3 (alpass allp2 (nth 6 icsc) (nth 8 ihz)))
    (setf alow  (lp allp3 ilowpass))
    (setf allp4 (alpass alow (nth 6 icsc) (nth 9 ihz)))
    (setf allp5 (alpass allp4 (nth 6 icsc) (nth 11 ihz)))

    allp5))
;; End of Reverb function.

(defun damp (s-in val)
				;make a mono copy of stereo input
	(let* ((mono (if (arrayp s-in)(mult 0.5(sum (aref s-in 0)(aref s-in 1))) s-in))
				;clip to 0 dB
				(mono (clip mono 1))
				;upper limit of filter
				(fmax (* 0.5 *sound-srate*))
				;lower limit of filter
				(fmin (+ 100(*(- 1 val)(- 1 val)(- fmax 100))))
				;make envelope for low-pass filter 
				(env (sum fmin (mult(- fmax fmin)(snd-avg mono 100 100 op-peak)))))
		(lp (lp (lp s-in fmin) env) env)))


(sim 
	(mult (- 1 mix) s)
	(mult mix (- 0.35 (* time 0.02))
		(highpass2
			(damp
				(reverb (stereo s width) time)
			damping)
		low-cut)))
		

